@@ -1,9 +1,7 @@
 import base64
-import hashlib
 import json
-import uuid
-from typing import Optional
 import os
+from typing import Optional
 
 import click
 import requests
@@ -53,7 +51,7 @@ def track_event(event_name: str, event_properties: dict) -> bool:
             return False
 
         base_url = get_base_login_url()
-        url = base_url+'/tensorfuse/track/tensorkube-event/'
+        url = base_url + '/tensorfuse/track/tensorkube-event/'
 
         body = {"id": user_email, "event": event_name, "properties": event_properties}
         x = requests.post(url, json=body)
@@ -69,7 +67,7 @@ def extract_workdir_from_dockerfile(dockerfile_path: str) -> Optional[str]:
         for line in lines:
             if line.startswith("WORKDIR"):
                 return line.split(" ")[1].replace("\n", "")
-    
+
     return None
 
 
@@ -80,10 +78,10 @@ def extract_command_from_dockerfile(dockerfile_path: str) -> Optional[str]:
         for line in lines:
             if line.startswith("CMD") or line.startswith("ENTRYPOINT"):
                 command = line.split(" ")[1:]
-    
+
     if not command:
         return None
-    
+
     command = " ".join(command)
     command = command.replace('[', "").replace(']', "").replace('"', '').replace(",", " ").replace("\n", "")
     return command
@@ -106,20 +104,21 @@ def verify_user(session_id: str = None, token: str = None, ) -> Optional[str]:
     if not (token and session_id):
         click.echo("Can't find tensorkube credentials.")
         return None
-    
+
     base_url = get_base_login_url()
-    response = requests.get( base_url+'/tensorfuse/verify-tensorkube-user/', 
-                 headers={'Content-Type': 'application/json'}, data=json.dumps({'session_id': session_id, 'token': token}))
+    response = requests.get(base_url + '/tensorfuse/verify-tensorkube-user/',
+                            headers={'Content-Type': 'application/json'},
+                            data=json.dumps({'session_id': session_id, 'token': token}))
     if response.status_code == 200:
         body = response.json()
         return body['email']
     else:
         return None
-    
+
 
 def save_user_credentials(session_id: str, token: str):
     token_path = os.path.expanduser('~/.tensorkube/token')
     os.makedirs(os.path.dirname(token_path), exist_ok=True)
     with open(token_path, "w") as f:
-        f.write("token="+token+"\n")
-        f.write("session_id="+session_id+"\n")
+        f.write("token=" + token + "\n")
+        f.write("session_id=" + session_id + "\n")
