@@ -82,12 +82,13 @@ def are_credentials_valid(credentials):
 
 
 # TODO!: make function generic to get any config value
-def get_bucket_name(env_name: Optional[str] = None):
+def get_bucket_name(env_name: Optional[str] = None, type: str = "build"):
     buckets = list_s3_buckets()
     if env_name:
-        prefix = f'{get_cluster_name()}-{env_name}-build-bucket-'
+        prefix = f'{get_cluster_name()}-{env_name}-{type}-bucket-'
     else:
-        prefix = f'{get_cluster_name()}-build-bucket-'
+        prefix = f'{get_cluster_name()}-{type}-bucket-'
+    # checks if such a bucket already exists
     for bucket in buckets:
         if bucket['Name'].startswith(prefix):
             return bucket['Name']
