@@ -1,46 +1,47 @@
-import os
 import time
-import uuid
-import requests
+from datetime import datetime, timezone
 
 import click
-from kubernetes import client
-from pkg_resources import resource_filename
 
-from tensorkube.constants import ADDON_NAME, REGION, Events, DEFAULT_NAMESPACE, get_cluster_name, BUILD_TOOL, PodStatus
+from tensorkube.constants import ADDON_NAME, REGION, Events, DEFAULT_NAMESPACE, get_cluster_name, CliColors
 from tensorkube.constants import get_mount_driver_role_name, get_mount_policy_name, get_base_login_url
-from tensorkube.helpers import create_mountpoint_driver_role_with_policy, sanitise_name, sanitise_assumed_role_arn, \
-    track_event, extract_command_from_dockerfile, extract_workdir_from_dockerfile, verify_user
-from tensorkube.tensorkube_token_server import run_server
+from tensorkube.helpers import create_mountpoint_driver_role_with_policy, sanitise_assumed_role_arn, track_event, \
+    sanitise_name
 from tensorkube.migration_service.migration_manager.migration_service import set_current_cli_version_to_cluster, \
     migrate_tensorkube
-from tensorkube.services.aws_service import get_aws_account_id, get_credentials, get_bucket_name, get_aws_user_arn, \
-    are_credentials_valid
+from tensorkube.services.aws_service import get_aws_account_id, get_credentials, get_bucket_name, get_aws_user_arn
 from tensorkube.services.cloudformation_service import delete_cloudformation_stack, delete_launch_templates, \
     cloudformation
+from tensorkube.services.dataset_service import upload_tensorkube_dataset, delete_tensorkube_dataset
+from tensorkube.services.deploy import Config, deploy_knative_service, DEFAULT_GPUS, DEFAULT_CPU, DEFAULT_MEMORY, \
+    DEFAULT_MIN_SCALE, DEFAULT_MAX_SCALE, DEFAULT_ENV, DEFAULT_SECRET, build_app, create_deployment_details
 from tensorkube.services.ecr_service import delete_all_tensorkube_ecr_repositories
 from tensorkube.services.eks_service import install_karpenter, apply_knative_crds, apply_knative_core, \
     delete_knative_crds, delete_knative_core, delete_karpenter_from_cluster, apply_nvidia_plugin, create_eks_addon, \
     delete_eks_addon, update_eks_kubeconfig
 from tensorkube.services.eksctl_service import create_base_tensorkube_cluster_eksctl, delete_cluster
-from tensorkube.services.environment_service import create_new_environment, list_environments, delete_environment, \
-    check_environment_exists
+from tensorkube.services.environment_service import create_new_environment, list_environments, delete_environment
 from tensorkube.services.filesystem_service import cleanup_filesystem_resources, configure_efs
 from tensorkube.services.iam_service import create_mountpoint_iam_policy, detach_role_policy, delete_role, delete_policy
 from tensorkube.services.istio import check_and_install_istioctl, install_istio_on_cluster, install_net_istio, \
     install_default_domain, remove_domain_server, uninstall_istio_from_cluster
-from tensorkube.services.k8s_service import apply_k8s_buildkit_config, create_aws_secret, create_build_pv_and_pvc, \
-    find_and_delete_old_job, check_pod_status, start_streaming_pod, start_streaming_service, get_image_tags_to_retain, \
-    apply_image_cleanup_job, get_pod_name_corresponing_to_job, get_pod_status_from_job, \
-    get_tensorkube_cluster_context_name
+from tensorkube.services.job_queue_service import deploy_job, create_cloud_resources_for_queued_job_support, queue_job, \
+    get_job_status, delete_all_job_resources, teardown_job_queue_support
+from tensorkube.services.k8s_service import create_aws_secret, create_build_pv_and_pvc, create_secret, list_secrets, \
+    delete_secret, list_keda_scaled_jobs
 from tensorkube.services.karpenter_service import apply_karpenter_configuration
 from tensorkube.services.knative_service import enable_knative_selectors_pv_pvc_capabilities, cleanup_knative_resources, \
-    delete_knative_services, apply_knative_service_with_podman, apply_virtual_service_for_routing
+    delete_knative_services
 from tensorkube.services.local_service import check_and_install_cli_tools
 from tensorkube.services.logging_service import configure_cloudwatch, teardown_cloudwatch
+from tensorkube.services.metapod_service import list_instances_in_cloud, start_devcontainer, purge_devcontainer, \
+    pause_devcontainer, reset_cloud
 from tensorkube.services.rich_service import list_tensorkube_deployments, describe_deployment, display_deployment_logs, \
-    ssh_into_deployed_service
-from tensorkube.services.s3_service import create_s3_bucket, delete_s3_bucket, upload_files_in_parallel, empty_s3_folder
+    ssh_into_deployed_service, display_secrets, list_tensorkube_datasets, display_job_logs, \
+    delete_tensorkube_job_by_prefix, list_tensorkube_training_jobs, display_keda_scaled_jobs
+from tensorkube.services.s3_service import create_s3_bucket, delete_s3_bucket
+from tensorkube.services.train import axolotl_train, get_job_prefix_from_job_id
+from tensorkube.tensorkube_token_server import run_server
 
 
 @click.group()
@@ -61,6 +62,11 @@ def deployment():
     pass
 
 
+@tensorkube.group()
+def secret():
+    pass
+
+
 @list.command()
 @click.option('--env', default=None, type=str, help='Environment to list the services from.')
 @click.option('--all', is_flag=True, help='List all services from all environments.')
@@ -69,6 +75,19 @@ def deployments(env, all, old):
     list_tensorkube_deployments(env_name=env, all=all, old=old)
 
 
+@list.command()
+@click.option('--env', default=DEFAULT_NAMESPACE, type=str, help='Environment to list the secrets from.')
+def secrets(env):
+    secrets = list_secrets(env)
+    if not secrets:
+        if env == DEFAULT_NAMESPACE:
+            click.echo("No secrets found")
+        else:
+            click.echo(f"No secrets found in environment " + click.style(env, bold=True))
+        return
+    display_secrets(secrets, env)
+
+
 @tensorkube.command()
 def init():
     click.echo("Initializing Tensorfuse runtime for your cloud...")
@@ -95,7 +114,7 @@ def configure():
     click.echo("Configuring the Tensorfuse runtime for your cloud...")
     start_time = time.time() * 1000
     can_track = track_event(Events.CONFIGURE_START.value, {"start_time": start_time})
-    if can_track == False:
+    if not can_track:
         click.echo("You need to login to Tensorfuse to configure Tensorkube.")
         click.echo("Please login using the command: " + click.style("tensorkube login", fg='cyan'))
         return
@@ -160,6 +179,9 @@ def configure():
     click.echo("Configuring EFS for the cluster...")
     configure_efs()
 
+    # install keda, create related resources
+    create_cloud_resources_for_queued_job_support()
+
     # set current cli version to the cluster
     set_current_cli_version_to_cluster()
 
@@ -189,6 +211,9 @@ def teardown():
     click.echo("Deleting all ECR repositories...")
     delete_all_tensorkube_ecr_repositories()
 
+    click.echo("Deleting all job queue resources...")
+    teardown_job_queue_support()
+
     # delete all services
     try:
         delete_knative_services()
@@ -302,133 +327,66 @@ def clear():
 
 
 @tensorkube.command()
-@click.option('--gpus', default=0, help='Number of GPUs needed for the service.')
-@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4'], case_sensitive=False), help='Type of GPU.')
-@click.option('--cpu', type=float, default=100, help='Number of CPU millicores. 1000 = 1 CPU')
-@click.option('--memory', type=float, default=200, help='Amount of RAM in megabytes.')
-@click.option('--min-scale', type=int, default=0, help='Minimum number of pods to run.')
-@click.option('--max-scale', type=int, default=3, help='Maximum number of pods to run.')
-@click.option('--env', default=None, type=str, help='Environment to deploy the service to.')
+@click.option('--gpus', default=DEFAULT_GPUS, help='Number of GPUs needed for the service.')
+@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S'], case_sensitive=False),
+              help='Type of GPU.')
+@click.option('--cpu', type=float, default=DEFAULT_CPU, help='Number of CPU millicores. 1000 = 1 CPU')
+@click.option('--memory', type=float, default=DEFAULT_MEMORY, help='Amount of RAM in megabytes.')
+@click.option('--min-scale', type=int, default=DEFAULT_MIN_SCALE, help='Minimum number of pods to run.')
+@click.option('--max-scale', type=int, default=DEFAULT_MAX_SCALE, help='Maximum number of pods to run.')
+@click.option('--env', default=DEFAULT_ENV, type=str, help='Environment to deploy the service to.')
 @click.option('--github-actions', is_flag=True, help='Deploying from Github Actions.')
-def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_actions):
+@click.option('--secret', default=DEFAULT_SECRET, type=str, multiple=True, help='Secret to use for the deployment.')
+@click.option('--config-file', type=str,
+              help='Path to the config.yaml file. you can use this file instead of cli flags. Keys in config.yaml take precedence than cli flags.')
+def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_actions, secret, config_file):
     """
     Deploy your containerized application on Tensorkube. This command requires
     a dockerfile to be present in the current directory.
     """
+    try:
+        config = Config(config_file)
+    except Exception as e:
+        click.echo(f"Failed to read config.yaml file: {e}")
+        return
+
+    config.set_if_not_exist('gpus', gpus).set_if_not_exist('gpu_type', gpu_type).set_if_not_exist('cpu',
+                                                                                                  cpu).set_if_not_exist(
+        'memory', memory).set_if_not_exist('min_scale', min_scale).set_if_not_exist('max_scale',
+                                                                                    max_scale).set_if_not_exist('env',
+                                                                                                                env).set_if_not_exist(
+        'github_actions', github_actions).set_if_not_exist('secret', secret)
+
+    # flags validation
+    if config.get('gpus') > 0:
+        if not config.get('gpu_type'):
+            click.echo(click.style("Error: GPU type is required when GPUs are specified.", fg=CliColors.ERROR.value))
+            return
+
     start_time = time.time() * 1000
     can_track = track_event(Events.DEPLOY_START.value, {"start_time": start_time})
-    if can_track == False:
+    if not can_track:
         click.echo("You need to login to Tensorfuse to run your deployment.")
         click.echo("Please login using the command: " + click.style("tensorkube login", fg='cyan'))
-        if github_actions:
+        if config.get('github_actions'):
             raise Exception("Please provide the necessary credentials to deploy using tensorkube")
         return
-    if env and not check_environment_exists(env_name=env):
-        click.echo("Environment does not exist. Please create the environment first.")
-        return
-    if gpus not in [0, 1, 4, 8]:
+    if config.get('gpus') not in [0, 1, 4, 8]:
         click.echo('Error: Invalid number of GPUs. Only supported values are 0, 1, 4, and 8.')
         return
-    cwd = os.getcwd()
-    is_dockerfile_present = False
-    for root, dirs, files in os.walk(cwd):
-        for file in files:
-            local_file = os.path.join(root, file)
-            if local_file == cwd + "/Dockerfile":
-                is_dockerfile_present = True
-    env_namespace = DEFAULT_NAMESPACE if not env else env
-    if not is_dockerfile_present:
-        click.echo("No Dockerfile found in current directory.")
-        return
-    else:
-        bucket_name = get_bucket_name(env_name=env)
-        non_sanitised_name = os.path.basename(cwd)
-        sanitised_project_name = sanitise_name(non_sanitised_name)
-        build_job_name = f'{BUILD_TOOL}-{sanitised_project_name}'
-        old_cleanup_job = 'cleanup-{}'.format(sanitised_project_name)
 
-        old_job_deleted = find_and_delete_old_job(job_name=build_job_name, namespace=env_namespace)
-        old_cleanup_job_del = find_and_delete_old_job(job_name=old_cleanup_job, namespace=env_namespace)
-        if not old_job_deleted:
-            click.echo("Another deployment is already in progress. Please wait for the build to complete.")
-            return
+    depl_secrets, env_namespace, image_tag, cwd, sanitised_project_name = create_deployment_details(secret=config.get('secret'),
+                                                                                                  env=config.get('env'))
+    pod_status = build_app(env=config.get('env'), image_tag=image_tag, sanitised_project_name=sanitised_project_name,
+                           upload_to_nfs=True)
+    deploy_knative_service(env=config.get('env'), gpus=config.get('gpus'),
+                           gpu_type=config.get('gpu_type'), cpu=config.get('cpu'), memory=config.get('memory'),
+                           min_scale=config.get('min_scale'), max_scale=config.get('max_scale'), pod_status=pod_status,
+                           cwd=cwd, sanitised_project_name=sanitised_project_name, image_tag=image_tag, secrets=depl_secrets)
 
-        # TODO!: add logic to update the aws-secret only if IAM Identity Center User
-        credentials = get_credentials()
-        if are_credentials_valid(credentials):
-            click.echo("Credentials are up to date")
-        else:
-            click.echo("The AWS credentials have expired. Please update the credentials.")
-            return
-
-        # TODO: figure out how to upload only the updated files to the s3 bucket
-        click.echo("Deleting any outdated files in the S3 bucket...")
-        empty_s3_folder(bucket_name=bucket_name, folder_name="build/" + sanitised_project_name + "/")
-
-        click.echo("Uploading the current directory to the S3 bucket...")
-        upload_files_in_parallel(bucket_name=bucket_name, folder_path=cwd,
-                                 s3_path="build/" + sanitised_project_name + "/")
-
-        click.echo("Building the Docker image...")
-        image_tag = uuid.uuid4().hex
-
-        apply_k8s_buildkit_config(sanitised_project_name=sanitised_project_name, image_tag=image_tag, env_name=env)
-        build_job_pod_name = get_pod_name_corresponing_to_job(job_name=build_job_name, namespace=env_namespace)
-
-        # TODO: stream multiple lines instead of one by one
-        start_streaming_pod(pod_name=build_job_pod_name, namespace=env_namespace, status=PodStatus.SUCCEEDED)
-
-        transition_time = time.time()
-        # wait for the pod to transition
-        while True:
-            try:
-                pod_status = check_pod_status(pod_name=build_job_pod_name, namespace=env_namespace)
-            except client.ApiException as e:
-                if e.status == 404:
-                    print('Pod not found.')
-                    pod_status = get_pod_status_from_job(job_name=build_job_name, namespace=env_namespace)
-                else:
-                    pod_status = PodStatus.FAILED.value
-            print('Waiting for pod to transition')
-            if pod_status in ['Succeeded', 'Completed', 'Failed']:
-                break
-            if time.time() - transition_time > 60:  # 60 seconds have passed
-                print("Timeout: Pod did not reach the desired state within 1 minute.")
-                break
-            time.sleep(5)
-
-        # check for build pod to succeed
-        if pod_status == 'Succeeded' or pod_status == 'Completed':
-            click.echo("Successfully built and pushed the Docker image.")
-            yaml_file_path = resource_filename('tensorkube', 'configurations/build_configs/knative_base_config.yaml')
-            dockerfile_path = cwd + "/Dockerfile"
-            workdir = extract_workdir_from_dockerfile(dockerfile_path)
-            command = extract_command_from_dockerfile(dockerfile_path)
-            service_name = f"{sanitised_project_name}-gpus-{gpus}-{str(gpu_type).lower()}"
-            apply_knative_service_with_podman(service_name=service_name, yaml_file_path=yaml_file_path,
-                                              sanitised_project_name=sanitised_project_name, image_tag=image_tag,
-                                              workdir=workdir, command=command, gpus=gpus, gpu_type=gpu_type, cpu=cpu,
-                                              memory=memory, min_scale=min_scale, max_scale=max_scale, env=env)
-            virtual_service_yaml_file_path = resource_filename('tensorkube',
-                                                               'configurations/build_configs/virtual_service.yaml')
-            apply_virtual_service_for_routing(service_name=service_name, yaml_file_path=virtual_service_yaml_file_path,
-                                              env=env, sanitised_project_name=sanitised_project_name)
-            end_time = time.time() * 1000
-            click.echo(
-                f"The service {service_name} sent for deployment. Waiting for it to become ready. Time taken: {(end_time - start_time) / 1000} ms.")
-            images_to_retain = get_image_tags_to_retain(sanitised_project_name=sanitised_project_name,
-                                                        service_name=service_name, namespace=env_namespace)
-            apply_image_cleanup_job(sanitised_project_name=sanitised_project_name, image_tags=images_to_retain, env=env)
-            start_streaming_service(service_name=service_name, namespace=env_namespace)
-            track_event(Events.DEPLOY_END.value,
-                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
-        else:
-            end_time = time.time() * 1000
-            track_event(Events.DEPLOY_ERROR.value,
-                        {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
-            click.echo(
-                "Failed to build the Docker image. Please check the logs for more details. Pod status: {}".format(
-                    pod_status))
+    end_time = time.time() * 1000
+    track_event(Events.DEPLOY_ERROR.value,
+                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time})
 
 
 @tensorkube.command()
@@ -528,10 +486,245 @@ def list():
 def login():
     click.echo("Opening the Tensorfuse dashboard in your default browser...")
     base_url = get_base_login_url()
-    click.launch( base_url + "/tensorfuse/login?tensorkube_login=True")
+    click.launch(base_url + "/tensorfuse/login?tensorkube_login=True")
     run_server()
 
 
+@secret.command()
+@click.option('--env', default=DEFAULT_NAMESPACE, type=str, help='Environment to create the secret in.')
+@click.option('--force', is_flag=True, help='Update the secret if one with the same neame already exists.')
+@click.argument('secret_name', type=str)
+@click.argument('key_value_pairs', nargs=-1)
+def create(secret_name, key_value_pairs, env, force):
+    try:
+        key_values = dict(pair.split('=') for pair in key_value_pairs)
+    except Exception as e:
+        click.echo(
+            click.style("Error: Invalid key-value pairs. Please provide key-value pairs in the format KEY=value.",
+                        fg=CliColors.ERROR.value))
+        return
+
+    click.echo("Creating secret " + click.style(secret_name, bold=True) + " in environment " + click.style(env,
+                                                                                                           bold=True) + " ...")
+
+    create_secret(name=secret_name, data=key_values, namespace=env, force=force)
+
+
+@secret.command()
+@click.option('--env', default=DEFAULT_NAMESPACE, type=str, help='Environment to delete the secret from.')
+@click.argument('secret_name', type=str)
+def delete(secret_name, env):
+    msg = "Are you sure you want to delete the secret? This action cannot be undone"
+    if click.confirm(click.style(msg, fg=CliColors.WARNING.value), abort=True):
+        click.echo("Deleting secret " + click.style(secret_name, bold=True) + " from environment " + click.style(env,
+                                                                                                                 bold=True) + " ...")
+
+        delete_secret(name=secret_name, namespace=env)
+
+
 @tensorkube.command()
 def test():
-    click.echo("Running tests...")
+    click.echo("Running tests... for someone")
+
+@tensorkube.command()
+@click.option('--cloud', default='aws', help='Cloud provider to use.')
+def reset(cloud):
+    reset_cloud(cloud)
+
+
+@tensorkube.group()
+@click.option('--cloud', default='aws', help='Cloud provider to use.')
+@click.pass_context
+def dev(ctx, cloud):
+    ctx.ensure_object(dict)
+    ctx.obj['cloud'] = cloud
+    time_now_utc = datetime.now(timezone.utc).isoformat()
+    can_track = track_event(Events.DEPLOY_START.value, {"start_time": time_now_utc})
+    if not can_track:
+        click.echo("You need to login to Tensorfuse to run your devcontainers.")
+        click.echo("Please login using the command: " + click.style("tensorkube login", fg='cyan'))
+        exit(0)
+
+
+@dev.command()
+@click.pass_context
+def list(ctx):
+    cloud = ctx.obj['cloud']
+    list_instances_in_cloud(cloud)
+
+
+@dev.command()
+@click.option('--gpu-type',
+              type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'A100', 'L40S', 'H100', 'None'], case_sensitive=False),
+              help='Type of GPU.')
+@click.option('--port', type=int, default=8080, help='Port to run the devcontainer on.')
+@click.pass_context
+def start(ctx, gpu_type: str, port: int):
+    cloud = ctx.obj['cloud']
+    if gpu_type == 'None':
+        gpu_type = None
+    start_devcontainer(cloud, gpu_type, port)
+
+
+@dev.command()
+@click.pass_context
+def stop(ctx):
+    cloud = ctx.obj['cloud']
+    pause_devcontainer(cloud)
+
+
+@dev.command()
+@click.pass_context
+def delete(ctx):
+    cloud = ctx.obj['cloud']
+    purge_devcontainer(cloud)
+
+
+@tensorkube.group()
+def train():
+    pass
+
+
+@train.command()
+@click.option('--gpus', default=DEFAULT_GPUS, help='Number of GPUs needed for the service.')
+@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S'], case_sensitive=False),
+              help='Type of GPU.')
+@click.option('--env', default=DEFAULT_ENV, type=str, help='Environment to deploy the service to.')
+@click.option('--job-id', default=None, type=str, help='Unique job id for the training job.')
+@click.option('--axolotl', is_flag=True, help='Run the axolotl training job.')
+@click.option('--config-path', default=None, type=str, help='Path to the axolotl config.yaml file.')
+@click.option('--secret', default=DEFAULT_SECRET, type=str, multiple=True, help='Secret to use for the deployment.')
+
+def create(gpus, gpu_type, env, job_id, axolotl, config_path, secret):
+    if axolotl:
+        axolotl_train(env=env, secrets=secret, gpus=gpus, gpu_type=gpu_type, job_id=job_id, config_path=config_path)
+    else:
+        click.echo("Invalid command. Please use the --axolotl flag to run the axolotl training job.")
+        return
+
+
+@train.command()
+@click.option('--job-id', required=True, type=str, help='Unique job id for the training job.')
+@click.option('--env', default=DEFAULT_ENV, type=str, help='Environment to deploy the service to.')
+def logs(job_id: str, env: str):
+    click.echo("Fetching logs for the training job...")
+    job_prefix = get_job_prefix_from_job_id(job_id)
+    namespace = env if env else DEFAULT_NAMESPACE
+    display_job_logs(job_prefix=job_prefix, namespace=namespace)
+
+
+@train.command()
+@click.option('--env', default=DEFAULT_ENV, type=str, help='Environment to delete the job from.')
+@click.option('--job-id', required=True, type=str, help='Unique job id for the training job.')
+def delete(job_id: str, env: str):
+    click.echo("Deleting the training job...")
+    job_prefix = get_job_prefix_from_job_id(job_id)
+    namespace = env if env else DEFAULT_NAMESPACE
+    delete_tensorkube_job_by_prefix(job_prefix=job_prefix, namespace=namespace)
+
+
+@train.command()
+@click.option('--env', default=DEFAULT_ENV, type=str, help='Environment to deploy the service to.')
+def list(env: str):
+    namespace = env if env else DEFAULT_NAMESPACE
+    # ax- for axolotl jobs
+    prefix = 'ax-'
+    list_tensorkube_training_jobs(namespace=namespace, job_prefix=prefix)
+
+
+@tensorkube.group()
+def datasets():
+    pass
+
+
+@datasets.command()
+def list():
+    list_tensorkube_datasets()
+
+
+@datasets.command()
+@click.option("--dataset-id", required=True, type=str, help="ID of the dataset to create.")
+@click.option("--path", required=True,
+              type=click.Path(exists=True, file_okay=True, dir_okay=False, path_type=str, readable=True),
+              callback=lambda ctx, param, value: value if value.endswith('.jsonl') else click.BadParameter(
+                  'File must be a .jsonl file'), help="Path to the JSONL file.")
+def create(dataset_id, path):
+    click.echo("Creating dataset...")
+    upload_tensorkube_dataset(dataset_id=dataset_id, file_path=path)
+
+
+@datasets.command()
+@click.option("--dataset-id", required=True, type=str, help="ID of the dataset to create.")
+def delete(dataset_id: str):
+    delete_tensorkube_dataset(dataset_id=dataset_id)
+
+
+
+@tensorkube.group()
+def job():
+    pass
+
+@job.command()
+@click.option('--name', required=True, type=str, help='Name of the job to deploy.')
+@click.option('--gpus', default=DEFAULT_GPUS, help='Number of GPUs needed for the service.')
+@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S'], case_sensitive=False),
+              help='Type of GPU.')
+@click.option('--cpu', type=float, default=DEFAULT_CPU, help='Number of CPU millicores. 1000 = 1 CPU')
+@click.option('--memory', type=str, default=DEFAULT_MEMORY, help='Amount of RAM in megabytes.')
+@click.option('--secret', default=DEFAULT_SECRET, type=str, multiple=True, help='Secret to use for the deployment.')
+@click.option('--max-scale', type=int, default=DEFAULT_MAX_SCALE, help='Maximum number of jobs to run concurrently.')
+def deploy(name, gpus, gpu_type, cpu, memory, secret, max_scale):
+    env = 'keda'
+
+    start_time = time.time() * 1000
+    can_track = track_event(Events.DEPLOY_START.value, {"start_time": start_time})
+    if not can_track:
+        click.echo("You need to login to Tensorfuse to run your deployment.")
+        click.echo("Please login using the command: " + click.style("tensorkube login", fg='cyan'))
+        return
+    if gpus not in [0, 1, 4, 8]:
+        click.echo('Error: Invalid number of GPUs. Only supported values are 0, 1, 4, and 8.')
+        return
+
+    depl_secrets, env_namespace, image_tag, cwd, sanitised_project_name = create_deployment_details(
+        secret=secret,
+        env=env
+    )
+    sanitised_job_name = sanitise_name(name)
+    pod_status = build_app(env=env, image_tag=image_tag, sanitised_project_name=sanitised_job_name,
+                           upload_to_nfs=True)
+
+    if pod_status == 'Succeeded' or pod_status == 'Completed':
+        deploy_job(env=env, job_name=name, gpus=gpus, gpu_type=gpu_type, cpu=cpu, memory=memory, max_scale=max_scale,
+                   sanitised_project_name=sanitised_job_name, image_tag=image_tag, secrets=depl_secrets, cwd=cwd)
+    else:
+        click.echo("Failed to build the Docker image. Please check the logs for more details. Pod status: {}".format(
+            pod_status))
+        return
+
+@job.command()
+@click.option("--job-name", required=True, type=str, help="Name of the job to delete.")
+def delete(job_name: str):
+    delete_all_job_resources(job_name)
+    click.echo("Job deleted successfully.")
+
+
+@job.command()
+@click.option("--job-name", required=True, type=str, help="Name of the job to delete.")
+@click.option("--job-id", required=True, type=str, help="Unique id for the job instance.")
+@click.option("--payload", required=True, type=str, help="Payload of the job to delete.")
+def queue(job_name: str, job_id:str, payload: str):
+    queue_job(job_name, job_id, payload)
+    click.echo("Job queued successfully.")
+
+@job.command()
+@click.option("--job-name", required=True, type=str, help="Name of the job to deploy.")
+@click.option("--job-id", required=True, type=str, help="Unique id of the job instance.")
+def get(job_name: str, job_id: str):
+    status = get_job_status(job_name, job_id)
+    print(status)
+
+@job.command()
+def list():
+    jobs = list_keda_scaled_jobs()
+    display_keda_scaled_jobs(jobs)
