@@ -104,6 +104,40 @@ def delete_karpenter_from_cluster():
     subprocess.run(command, check=True)
 
 
+def install_keda(sqs_access_role_arn:str):
+
+    helm_repo_add_command = ["helm", "repo", "add", "kedacore", "https://kedacore.github.io/charts"]
+    try:
+        subprocess.run(helm_repo_add_command, check=True)
+    except Exception as e:
+        print(f"Error while adding helm repo: {e}")
+        return False
+
+    helm_repo_update_command = ["helm", "repo", "update"]
+    try:
+        subprocess.run(helm_repo_update_command, check=True)
+    except Exception as e:
+        print(f"Error while updating helm repo: {e}")
+        return False
+
+    install_command = ["helm", "install", "keda", "kedacore/keda", "--namespace", "keda", "--create-namespace",
+                       "--set", "podIdentity.aws.irsa.enabled=true",
+                       "--set", f"podIdentity.aws.irsa.roleArn={sqs_access_role_arn}"]
+
+    try:
+        subprocess.run(install_command, check=True)
+        return True
+    except Exception as e:
+        print(f"Error while installing keda: {e}")
+        return False
+
+
+def delete_keda_from_cluster():
+    # helm uninstall keda -n keda
+    command = ["helm", "uninstall", "keda", "-n", "keda"]
+    subprocess.run(command, check=True)
+
+
 def update_eks_kubeconfig(region: str = REGION):
     command = ["aws", "eks", "update-kubeconfig", "--name", get_cluster_name(), "--region", region]
     subprocess.run(command, check=True)
@@ -200,8 +234,7 @@ def create_eks_addon(cluster_name, addon_name, account_no, mountpoint_driver_rol
             raise e
 
 
-def delete_eks_addon( addon_name, region=REGION):
-    cluster_name = get_cluster_name()
+def delete_eks_addon(cluster_name: str, addon_name:str, region:str=REGION):
     client = boto3.client('eks', region_name=region)
     try:
         # Attempt to delete the addon
