@@ -4,7 +4,7 @@ from typing import Optional
 import boto3
 import click
 import yaml
-from kubernetes import config, client
+from kubernetes import config, client, utils
 from pkg_resources import resource_filename
 
 from tensorkube.constants import NAMESPACE, REGION, get_efs_service_account_name, get_efs_role_name, get_cluster_name, \
@@ -14,7 +14,7 @@ from tensorkube.services.eks_service import get_cluster_oidc_issuer_url, get_eks
     get_security_group_id_by_name
 from tensorkube.services.iam_service import delete_iam_role
 from tensorkube.services.k8s_service import delete_pvc_using_name_and_namespace, delete_pv_using_name, \
-    get_tensorkube_cluster_context_name
+    get_tensorkube_cluster_context_name, get_efs_claim_name
 
 
 def get_efs_name(cluster_name: str, env: Optional[str] = None):
@@ -467,3 +467,30 @@ def cleanup_filesystem_resources():
     delete_efs_filesystem(f'{get_cluster_name()}-efs')
     delete_security_group(get_efs_security_group_name())
     delete_iam_role(get_efs_role_name())
+
+
+def delete_efs_directory_for_deployment(sanitised_project_name: str, env: Optional[str] = None,
+                            context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+    k8s_api_client = config.new_client_from_config(context=context_name)
+
+    cleanup_config_file_path = resource_filename('tensorkube', 'configurations/build_configs/efs_cleanup_pod.yaml')
+    with open(cleanup_config_file_path) as f:
+        cleanup_config = yaml.safe_load(f)
+    cleanup_config['metadata']['name'] = 'cleanup-{}'.format(sanitised_project_name)
+    namespace_to_use = env if env else DEFAULT_NAMESPACE  # Replace 'default' with your default namespace if needed
+    cleanup_config['metadata']['namespace'] = namespace_to_use
+
+    for volume in cleanup_config['spec']['template']['spec']['volumes']:
+        if volume['name'] == 'efs-pvc':
+            volume['persistentVolumeClaim']['claimName'] = get_efs_claim_name(env_name=env)
+
+    cleanup_config['spec']['template']['spec']['containers'][0]['command'] = ["/bin/sh", "-c",
+                                                                              f"""rm -rf /mnt/efs/images/{sanitised_project_name}
+        echo 'Deletion completed' """]
+
+    utils.create_from_dict(k8s_api_client, cleanup_config)
+    click.echo('Deployed a delete config job')
