@@ -3,17 +3,13 @@ import json
 import boto3
 from botocore.exceptions import ClientError
 from pkg_resources import resource_filename
+from typing import Dict
 
-from tensorkube.constants import REGION
+from tensorkube.constants import REGION, get_cluster_name
+from tensorkube.services.aws_service import get_aws_account_id
 
 
-def create_mountpoint_iam_policy(policy_name, bucket_name, region=REGION):
-    policy_file_path = resource_filename('tensorkube', 'configurations/aws_configs/mountpoint_policy.json')
-    with open(policy_file_path, 'r') as f:
-        policy = json.load(f)
-    for statement in policy['Statement']:
-        statement['Resource'] = [r.replace('USER_BUCKET', bucket_name) for r in statement['Resource']]
-
+def create_iam_policy(policy_name: str, policy_document: Dict, region: str = REGION):
     iam = boto3.client('iam', region_name=region)
     sts = boto3.client('sts')
     try:
@@ -25,7 +21,7 @@ def create_mountpoint_iam_policy(policy_name, bucket_name, region=REGION):
     except ClientError as e:
         if e.response['Error']['Code'] == 'NoSuchEntity':
             print(f"IAM policy {policy_name} does not exist. Proceeding with creation.")
-            response = iam.create_policy(PolicyName=policy_name, PolicyDocument=json.dumps(policy), )
+            response = iam.create_policy(PolicyName=policy_name, PolicyDocument=json.dumps(policy_document), )
             print(f"IAM policy {policy_name} created successfully.")
             return response
         else:
@@ -33,12 +29,44 @@ def create_mountpoint_iam_policy(policy_name, bucket_name, region=REGION):
             raise e
 
 
-def create_s3_csi_driver_role(account_no: str, role_name: str, oidc_issuer_url: str, namespace: str,
-                              service_account_name: str):
+def create_mountpoint_iam_policy(policy_name, bucket_name, region=REGION):
+    policy_file_path = resource_filename('tensorkube', 'configurations/aws_configs/mountpoint_policy.json')
+    with open(policy_file_path, 'r') as f:
+        policy = json.load(f)
+    for statement in policy['Statement']:
+        statement['Resource'] = [r.replace('USER_BUCKET', bucket_name) for r in statement['Resource']]
+
+    return create_iam_policy(policy_name, policy, region)
+
+
+def create_sqs_access_policy(policy_name, region=REGION):
+    policy_file_path = resource_filename('tensorkube', 'configurations/aws_configs/sqs_access_policy.json')
+    with open(policy_file_path, 'r') as f:
+        policy = json.load(f)
+    for statement in policy['Statement']:
+        statement['Resource'] = f"arn:aws:sqs:{region}:{get_aws_account_id()}:{get_cluster_name()}-*"
+    return create_iam_policy(policy_name, policy, region)
+
+def create_dynamo_access_policy(policy_name, region=REGION):
+    policy_file_path = resource_filename('tensorkube', 'configurations/aws_configs/dynamo_access_policy.json')
+    with open(policy_file_path, 'r') as f:
+        policy = json.load(f)
+    for statement in policy['Statement']:
+        resource = statement['Resource']
+        resource = resource.replace('REGION', region)
+        resource = resource.replace('ACCOUNT_NO', get_aws_account_id())
+        resource = resource.replace('TABLE_NAME', f"{get_cluster_name()}-*")
+        statement['Resource'] = resource
+    return create_iam_policy(policy_name, policy, region)
+
+
+
+def create_iam_role_with_service_account_cluster_access(account_no: str, oidc_issuer_url: str, role_name: str,
+                                                        service_account_name: str, namespace: str, region: str=REGION):
     oidc_issuer = oidc_issuer_url[8:]
     region = oidc_issuer.split('.')[2]
     trust_policy_file_path = resource_filename('tensorkube',
-                                               'configurations/aws_configs/aws_s3_csi_driver_trust_policy.json')
+                                               'configurations/aws_configs/iam_role_cluster_access_trust_policy.json')
     with open(trust_policy_file_path, 'r') as f:
         trust_policy = json.load(f)
     trust_policy['Statement'][0]['Principal']['Federated'] = 'arn:aws:iam::{}:oidc-provider/{}'.format(account_no,
@@ -51,8 +79,9 @@ def create_s3_csi_driver_role(account_no: str, role_name: str, oidc_issuer_url:
 
     try:
         # Check if the IAM role already exists
-        iam.get_role(RoleName=role_name)
+        response = iam.get_role(RoleName=role_name)
         print(f"IAM role {role_name} already exists. Skipping creation.")
+        return response
     except ClientError as e:
         if e.response['Error']['Code'] == 'NoSuchEntity':
             print(f"IAM role {role_name} does not exist. Proceeding with creation.")
@@ -64,6 +93,18 @@ def create_s3_csi_driver_role(account_no: str, role_name: str, oidc_issuer_url:
             raise e
 
 
+def create_s3_csi_driver_role(account_no: str, role_name: str, oidc_issuer_url: str, namespace: str,
+                              service_account_name: str):
+    create_iam_role_with_service_account_cluster_access(account_no, oidc_issuer_url, role_name, service_account_name,
+                                                        namespace)
+
+
+def create_sqs_access_role(account_no: str, oidc_issuer_url: str, role_name: str, namespace: str,
+                           service_account_name: str):
+    return create_iam_role_with_service_account_cluster_access(account_no, oidc_issuer_url, role_name, service_account_name,
+                                                        namespace)
+
+
 def attach_role_policy(account_no, policy_name, role_name, region=REGION):
     client = boto3.client('iam', region_name=region)
     response = client.attach_role_policy(PolicyArn='arn:aws:iam::{}:policy/{}'.format(account_no, policy_name),
@@ -102,3 +143,12 @@ def delete_iam_role(role_name):
     # Delete the role
     iam_client.delete_role(RoleName=role_name)
     print(f"IAM Role {role_name} deleted")
+
+
+def get_role_name_for_prefix(prefix: str):
+    iam = boto3.client('iam')
+    roles = iam.list_roles()['Roles']
+    for role in roles:
+        if role['RoleName'].startswith(prefix):
+            return role['RoleName']
+    return None
