@@ -1,7 +1,7 @@
 import subprocess
 import time
 from threading import Thread
-from typing import List, Optional
+from typing import List, Optional, Dict
 
 import click
 import yaml
@@ -10,8 +10,9 @@ from kubernetes.client import ApiTypeError, ApiException
 from pkg_resources import resource_filename
 
 from tensorkube.constants import NAMESPACE, REGION, PodStatus, BUILD_TOOL, DEFAULT_NAMESPACE, TENSORFUSE_NAMESPACES, \
-    get_cluster_name
-
+    get_cluster_name, CliColors
+from tensorkube.services.aws_service import get_aws_account_id
+import json
 
 def get_s3_pv_name(env_name: Optional[str] = None):
     if env_name:
@@ -166,34 +167,23 @@ def create_build_pv_and_pvc(bucket_name: str, region: str = REGION, env: Optiona
             raise e
 
 
-def apply_k8s_buildkit_config(sanitised_project_name: str, image_tag: str, region: str = REGION,
-                              env_name: Optional[str] = None, context_name: Optional[str] = None):
-    if not context_name:
-        context_name = get_tensorkube_cluster_context_name()
-        if not context_name:
-            return None
-    k8s_api_client = config.new_client_from_config(context=context_name)
 
-    buildkit_config_file_path = resource_filename('tensorkube', 'configurations/build_configs/buildkit.yaml')
-    with open(buildkit_config_file_path) as f:
-        buildkit_config = yaml.safe_load(f)
+def get_buildkit_command(sanitised_project_name: str, image_tag: str, image_url: Optional[str] =None, upload_to_nfs: bool= False, region: str = REGION):
+    aws_account_number =  get_aws_account_id()
+    command = [ "/bin/sh", "-c",
+        f"""
+        apk add --no-cache curl unzip aws-cli docker
+        aws ecr get-login-password --region {region} | docker login --username AWS --password-stdin {aws_account_number}.dkr.ecr.{region}.amazonaws.com
+        buildctl-daemonless.sh build\
+            --frontend dockerfile.v0\
+            --local context=/data/build/{sanitised_project_name}\
+            --local dockerfile=/data/build/{sanitised_project_name}\
+            --output type=image,name={image_url},push=true\
+        """
+    ]
 
-    buildkit_config['metadata']['name'] = 'buildkit-{}'.format(sanitised_project_name)
-    buildkit_config['spec']['template']['spec']['containers'][0]['env'][0]['value'] = region
-
-    # Include the namespace in the buildkit configuration
-    # Replace 'default' with your default namespace if needed
-    namespace_to_use = env_name if env_name else DEFAULT_NAMESPACE
-    buildkit_config['metadata']['namespace'] = namespace_to_use
-
-    # Modify volume claim names based on env_name
-    for volume in buildkit_config['spec']['template']['spec']['volumes']:
-        if volume['name'] == 'persistent-storage':
-            volume['persistentVolumeClaim']['claimName'] = get_s3_claim_name(env_name=env_name)
-        elif volume['name'] == 'efs-pvc':
-            volume['persistentVolumeClaim']['claimName'] = get_efs_claim_name(env_name=env_name)
-
-    buildkit_config['spec']['template']['spec']['containers'][0]['command'] = ["/bin/sh", "-c", f"""
+    if upload_to_nfs:
+        command =  ["/bin/sh", "-c", f"""
         mkdir -p /data/cache
         mkdir -p /test/tars/{sanitised_project_name} /mnt/efs/images/{sanitised_project_name}/{image_tag}/rootfs /test/tars/{sanitised_project_name} /test/images/{sanitised_project_name}/rootfs
         time buildctl-daemonless.sh build\
@@ -252,6 +242,38 @@ def apply_k8s_buildkit_config(sanitised_project_name: str, image_tag: str, regio
         process_batches "file"
         process_batches "link"
         echo "Uploaded rootfs to NFS" """]
+    return command
+
+        
+
+def apply_k8s_buildkit_config(sanitised_project_name: str, image_tag: str, region: str = REGION,
+                              env_name: Optional[str] = None, context_name: Optional[str] = None, image_url: Optional[str] = None, upload_to_nfs: bool = False):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+    k8s_api_client = config.new_client_from_config(context=context_name)
+
+    buildkit_config_file_path = resource_filename('tensorkube', 'configurations/build_configs/buildkit.yaml')
+    with open(buildkit_config_file_path) as f:
+        buildkit_config = yaml.safe_load(f)
+
+    buildkit_config['metadata']['name'] = 'buildkit-{}'.format(sanitised_project_name)
+    buildkit_config['spec']['template']['spec']['containers'][0]['env'][0]['value'] = region
+
+    # Include the namespace in the buildkit configuration
+    # Replace 'default' with your default namespace if needed
+    namespace_to_use = env_name if env_name else DEFAULT_NAMESPACE
+    buildkit_config['metadata']['namespace'] = namespace_to_use
+
+    # Modify volume claim names based on env_name
+    for volume in buildkit_config['spec']['template']['spec']['volumes']:
+        if volume['name'] == 'persistent-storage':
+            volume['persistentVolumeClaim']['claimName'] = get_s3_claim_name(env_name=env_name)
+        elif volume['name'] == 'efs-pvc':
+            volume['persistentVolumeClaim']['claimName'] = get_efs_claim_name(env_name=env_name)
+
+    buildkit_config['spec']['template']['spec']['containers'][0]['command'] = get_buildkit_command(sanitised_project_name, image_tag, image_url, upload_to_nfs, region)
 
     utils.create_from_dict(k8s_api_client, buildkit_config)
     click.echo('Deployed a Buildkit image')
@@ -884,3 +906,290 @@ def remove_invalid_contexts(tensorkube_contexts: List[str]) -> Optional[str]:
     elif len(tensorkube_contexts) > 1:
         return tensorkube_contexts[0]
     return None
+
+
+def create_secret(name: str, namespace: str, data: Dict[str, str], force: bool = False, context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            raise Exception("No context found for the cluster. Please configure Tensorfuse properly.")
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    v1 = client.CoreV1Api(k8s_api_client)
+ 
+    secret = client.V1Secret()
+    secret.metadata = client.V1ObjectMeta(name=name, namespace=namespace)
+    secret.string_data = data
+
+
+    try:
+        # Check if the secret already exists
+        v1.read_namespaced_secret(name, namespace)
+        if force:
+            # If the secret exists and force is True, update the secret
+            v1.replace_namespaced_secret(name, namespace, body=secret)
+            click.echo(click.style("Updated already existing secret ", fg=CliColors.SUCCESS.value) 
+                       + click.style(name, bold=True, fg=CliColors.SUCCESS.value)
+                       + click.style(" in namespace ", fg=CliColors.SUCCESS.value) 
+                       + click.style(namespace, bold=True, fg=CliColors.SUCCESS.value))
+        else:
+            click.echo(click.style("Secret ", fg=CliColors.WARNING.value) 
+                       + click.style(name,  bold=True, fg=CliColors.WARNING.value)
+                       + click.style(" already exists in namespace ", fg=CliColors.WARNING.value)
+                       + click.style(namespace, bold=True, fg=CliColors.WARNING.value)
+                       + click.style(". Use --force to update the secret.", fg=CliColors.WARNING.value))
+    except ApiException as e:
+        if e.status == 404:
+            # If the secret does not exist, create it
+            try:
+                v1.create_namespaced_secret(namespace=namespace, body=secret)
+                click.echo(click.style("Secret ", fg=CliColors.SUCCESS.value) 
+               + click.style(name, bold=True, fg=CliColors.SUCCESS.value)
+               + click.style(" created successfully.", fg=CliColors.SUCCESS.value))
+            except ApiException as e:
+                click.echo(click.style("An error occurred while creating the secret", fg=CliColors.ERROR.value))
+                click.echo(click.style(f"Error: {e}", fg=CliColors.ERROR.value))
+        else:
+            raise e
+
+def list_secrets(namespace: str, context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return []
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    v1 = client.CoreV1Api(k8s_api_client)
+    secrets = v1.list_namespaced_secret(namespace).items
+    if not secrets:
+        return []
+    return secrets
+
+
+def delete_secret(name: str, namespace: str, context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            return None
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    v1 = client.CoreV1Api(k8s_api_client)
+    try:
+        v1.delete_namespaced_secret(name, namespace)
+        click.echo(click.style("Secret ", fg=CliColors.SUCCESS.value) 
+                   + click.style(name, bold=True, fg=CliColors.SUCCESS.value)
+                   + click.style(" deleted successfully.", fg=CliColors.SUCCESS.value))
+    except ApiException as e:
+        if e.status == 404:
+            click.echo(click.style("Secret ", fg=CliColors.WARNING.value) 
+                       + click.style(name, bold=True, fg=CliColors.WARNING.value)
+                       + click.style(" not found in namespace ", fg=CliColors.WARNING.value) 
+                       + click.style(namespace, bold=True, fg=CliColors.WARNING.value))
+        else:
+            raise e
+
+
+def create_configmap(name: str, namespace: str, data: Dict[str, str], force: bool = False, context_name: Optional[str] = None):
+    if not context_name:
+        context_name = get_tensorkube_cluster_context_name()
+        if not context_name:
+            raise Exception("No context found for the cluster. Please configure Tensorfuse properly.")
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    v1 = client.CoreV1Api(k8s_api_client) 
+    configmap = client.V1ConfigMap()
+    configmap.metadata = client.V1ObjectMeta(name=name, namespace=namespace)
+    configmap.data = {"config": yaml.dump(data)}
+    try:
+        # Check if the configmap already exists
+        v1.read_namespaced_config_map(name, namespace)
+        if force:
+            # If the configmap exists and force is True, update the configmap
+            v1.replace_namespaced_config_map(name, namespace, body=configmap)
+    except ApiException as e:
+        if e.status == 404:
+            # If the configmap does not exist, create it
+            try:
+                v1.create_namespaced_config_map(namespace=namespace, body=configmap)
+            except ApiException as e:
+                raise e
+        else:
+            raise e
+
+def get_pods_for_jobs(job_name: str, namespace: str = "default"):
+    context_name = get_tensorkube_cluster_context_name()
+    if not context_name:
+        return None
+    k8s_api_client = config.new_client_from_config(context=context_name)
+    api = client.CoreV1Api(k8s_api_client)
+    try:
+        return api.list_namespaced_pod(
+            namespace=namespace,
+            label_selector=f"job-name={job_name}"
+        )
+    except ApiException as e:
+        print(f"Exception when calling CoreV1Api->list_namespaced_pod: {e}")
+        return None
+
+
+def list_jobs(namespace: Optional[str] = None, all: bool = False,
+              job_name_prefix: Optional[str] = None):
+    """
+    Lists Kubernetes jobs in a namespace or across all namespaces with optional job name prefix filtering
+
+    Args:
+        namespace: Specific namespace to list jobs from
+        all: If True, lists jobs from all namespaces
+        context_name: Kubernetes context name to use
+        job_name_prefix: Optional prefix to filter job names
+
+    Returns:
+        Dict containing list of jobs or None if context not found
+    """
+    context_name = get_tensorkube_cluster_context_name()
+    if not context_name:
+        return None
+
+    api_client = config.new_client_from_config(context=context_name)
+    batch_v1 = client.BatchV1Api(api_client)
+
+    try:
+        if all:
+            job_list = batch_v1.list_job_for_all_namespaces()
+        else:
+            namespace = namespace if namespace else DEFAULT_NAMESPACE
+            job_list = batch_v1.list_namespaced_job(namespace=namespace)
+
+        # Filter jobs by prefix if specified
+        if job_name_prefix:
+            filtered_items = [
+                job for job in job_list.items
+                if job.metadata.name.startswith(job_name_prefix)
+            ]
+            job_list.items = filtered_items
+
+        return job_list
+    except ApiException as e:
+        print(f"Exception when calling BatchV1Api: {e}")
+        return None
+
+
+def delete_job(job_name: str, namespace: str = "default") -> bool:
+    """
+    Stops and deletes a Kubernetes job and its associated pods
+
+    Args:
+        job_name: Name of the job to delete
+        namespace: Kubernetes namespace where the job exists
+
+    Returns:
+        bool: True if deletion was successful, False otherwise
+    """
+    try:
+        # Get API clients
+        api_client = config.new_client_from_config(context=get_tensorkube_cluster_context_name())
+        batch_v1 = client.BatchV1Api(api_client)
+        core_v1 = client.CoreV1Api(api_client)
+
+        # First check if job exists
+        try:
+            batch_v1.read_namespaced_job(name=job_name, namespace=namespace)
+        except ApiException as e:
+            if e.status == 404:
+                click.echo(click.style(f"Job {job_name} not found in namespace {namespace}", fg='red'))
+                return False
+            raise e
+
+        # Delete associated pods first
+        try:
+            pods = core_v1.list_namespaced_pod(
+                namespace=namespace,
+                label_selector=f"job-name={job_name}"
+            )
+            for pod in pods.items:
+                core_v1.delete_namespaced_pod(
+                    name=pod.metadata.name,
+                    namespace=namespace
+                )
+        except ApiException as e:
+            print(f"Error deleting pods for job {job_name}: {e}")
+
+        # Delete the job
+        try:
+            batch_v1.delete_namespaced_job(
+                name=job_name,
+                namespace=namespace,
+                body=client.V1DeleteOptions(
+                    propagation_policy='Foreground',
+                    grace_period_seconds=0
+                )
+            )
+            click.echo(click.style(f"Successfully deleted job {job_name}", fg='green'))
+            return True
+        except ApiException as e:
+            click.echo(click.style(f"Error deleting job {job_name}: {e}", fg='red'))
+            return False
+
+    except Exception as e:
+        print(f"Error in delete_job: {e}")
+        click.echo(click.style(f"Failed to delete job: {str(e)}", fg='red'))
+        return False
+
+
+def list_keda_scaled_jobs():
+    context_name = get_tensorkube_cluster_context_name()
+    if not context_name:
+        return None
+
+    api_client = config.new_client_from_config(context=context_name)
+    k8s_client = client.CustomObjectsApi(api_client)
+
+    try:
+        keda_scaled_jobs = k8s_client.list_namespaced_custom_object(
+            group="keda.sh",
+            version="v1alpha1",
+            plural="scaledjobs",
+            namespace="keda"
+        )
+        return keda_scaled_jobs
+    except ApiException as e:
+        print(f"Exception when calling CustomObjectsApi: {e}")
+        return None
+
+
+def list_trigger_authentications():
+    context_name = get_tensorkube_cluster_context_name()
+    if not context_name:
+        return None
+
+    api_client = config.new_client_from_config(context=context_name)
+    k8s_client = client.CustomObjectsApi(api_client)
+
+    try:
+        trigger_authentications = k8s_client.list_namespaced_custom_object(
+            group="keda.sh",
+            version="v1alpha1",
+            plural="triggerauthentications",
+            namespace="keda"
+        )
+        return trigger_authentications
+    except ApiException as e:
+        print(f"Exception when calling CustomObjectsApi: {e}")
+        return None
+
+
+def delete_trigger_authentication(trigger_auth_name: str):
+    context_name = get_tensorkube_cluster_context_name()
+    if not context_name:
+        return None
+
+    api_client = config.new_client_from_config(context=context_name)
+    k8s_client = client.CustomObjectsApi(api_client)
+
+    try:
+        k8s_client.delete_namespaced_custom_object(
+            group="keda.sh",
+            version="v1alpha1",
+            plural="triggerauthentications",
+            namespace="keda",
+            name=trigger_auth_name
+        )
+    except ApiException as e:
+        print(f"Exception when calling CustomObjectsApi: {e}")
+        return None
