@@ -1,7 +1,7 @@
 import subprocess
 import time
 from datetime import datetime
-from typing import Optional
+from typing import Optional, List
 
 import click
 import yaml
@@ -24,12 +24,14 @@ def get_instance_family_from_gpu_type(gpu_type):
         return 'g5g'
     elif gpu_type == 'l4':
         return 'g6'
+    elif gpu_type == 'l40s':
+        return 'g6e'
     else:
         raise ValueError(f"Unsupported GPU type: {gpu_type}")
 
 
 def get_supported_gpu_families():
-    return ['p3', 'g5', 'g4dn', 'g5g', 'g6']
+    return ['p3', 'g5', 'g4dn', 'g5g', 'g6', 'g6e']
 
 
 def apply_anti_affinity_for_gpus(yaml_dict):
@@ -102,7 +104,8 @@ def apply_virtual_service_for_routing(service_name: str, yaml_file_path: str, sa
 def apply_knative_service_with_podman(service_name: str, yaml_file_path: str, sanitised_project_name: str,
                                       image_tag: str, workdir: str, command: str, gpus: int, gpu_type: str,
                                       env: Optional[str] = None, cpu: float = 100, memory: int = 200,
-                                      min_scale: int = 0, max_scale: int = 3, context_name: Optional[str] = None):
+                                      min_scale: int = 0, max_scale: int = 3, context_name: Optional[str] = None,
+                                      secrets: List[str] = []):
     # Load kube config
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
@@ -124,6 +127,25 @@ def apply_knative_service_with_podman(service_name: str, yaml_file_path: str, sa
     for volume in yaml_dict['spec']['template']['spec']['volumes']:
         if volume['name'] == 'efs-storage':
             volume['persistentVolumeClaim']['claimName'] = get_efs_claim_name(env_name=env)
+    
+    if secrets:
+        yaml_dict['spec']['template']['spec']['volumes'].append({
+            'name': 'secrets',
+            'projected': {
+                'sources': [{
+                    'secret': {
+                        'name': secret_name
+                    }
+                } for secret_name in secrets]
+            }
+        })
+
+        yaml_dict['spec']['template']['spec']['containers'][0]['volumeMounts'].append({
+            'name': 'secrets',
+            'mountPath': '/mnt/secrets',
+            'readOnly': True
+        })
+
 
     if gpus > 0:
         yaml_dict['spec']['template']['spec']['containers'][0]['image'] = "tensorfuse/podman-nvidia:v1"
@@ -141,22 +163,66 @@ def apply_knative_service_with_podman(service_name: str, yaml_file_path: str, sa
     else:
         final_command = command
 
+    if secrets:
+        secrets_to_env_vars_command = """\
+folder_path="/mnt/secrets"
+
+# Initialize an empty string to hold the environment variables
+env_vars=""
+
+# Loop through each file in the folder
+for file in "$folder_path"/*; do
+  if [[ -f $file ]]; then
+    # Get the filename without the path
+    filename=$(basename "$file")
+    
+    # Get the contents of the file
+    contents=$(<"$file")
+    
+    # Escape the contents to handle any special characters or spaces
+    escaped_contents=$(printf '%q' "$contents")
+
+    # Append to the env_vars string in the format --env filename=contents
+    env_vars="$env_vars --env $filename=$escaped_contents"
+  fi
+done"""
+    else:
+        secrets_to_env_vars_command = "env_vars=''"
+
     yaml_dict['spec']['template']['spec']['containers'][0]['command'] = ["/bin/sh", "-c",
                                                                          f"""{config_nvidia_ctk_commands}
+{secrets_to_env_vars_command}
         sed -i 's|mount_program = "/usr/bin/fuse-overlayfs"|mount_program = ""|' /etc/containers/storage.conf
-        sudo podman run --name mycontainer {podman_gpu_tags} --network=host \
+        sudo podman run --name mycontainer $env_vars {podman_gpu_tags} --network=host \
          --rootfs /mnt/efs/images/{sanitised_project_name}/{image_tag}/rootfs:O sh -c "{final_command}" """]
     if gpus > 0:
         yaml_dict['spec']['template']['spec']['nodeSelector'] = {
             'karpenter.k8s.aws/instance-family': get_instance_family_from_gpu_type(gpu_type)}
+        yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/scale-to-zero-pod-retention-period'] = '10m'
+        yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/scale-down-delay'] = '10m'
+        yaml_dict['spec']['template']['spec']['containers'][0]['volumeMounts'].append({
+            'name': 'dshm',
+            'mountPath': '/dev/shm'
+        })
+        yaml_dict['spec']['template']['spec']['volumes'].append({
+            'name': 'dshm',
+            'emptyDir': {
+                'medium': 'Memory',
+                'sizeLimit': '10Gi'
+            }
+        })
     else:
         yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests'][
             'memory'] = f'{str(int(memory))}M'
         yaml_dict['spec']['template']['spec']['containers'][0]['resources']['requests']['cpu'] = f'{str(int(cpu))}m'
         apply_anti_affinity_for_gpus(yaml_dict)
+    if gpus > 2:
+        yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/scale-to-zero-pod-retention-period'] = '15m'
+        yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/scale-down-delay'] = '15m'
     # apply min scale and max scale arguements
     yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/min-scale'] = str(min_scale)
     yaml_dict['spec']['template']['metadata']['annotations']['autoscaling.knative.dev/max-scale'] = str(max_scale)
+    
 
     yaml_dict['spec']['template']['metadata']['annotations']['image_tag'] = image_tag
     yaml_dict['spec']['template']['metadata']['annotations']['deploy_time'] = datetime.now().isoformat()
@@ -260,9 +326,15 @@ def get_knative_service(service_name: str, namespace: str = "default", context_n
             return None
     k8s_api_client = config.new_client_from_config(context=context_name)
     api = client.CustomObjectsApi(k8s_api_client)
-    ksvc = api.get_namespaced_custom_object(group="serving.knative.dev", version="v1", plural="services",
+    try:
+        ksvc = api.get_namespaced_custom_object(group="serving.knative.dev", version="v1", plural="services",
                                             name=service_name, namespace=namespace, )
-    return ksvc
+        return ksvc
+    except ApiException as e:
+        if e.status == 404:
+            return None
+        else:
+            raise e
 
 
 def get_ready_condition(service):
@@ -274,6 +346,8 @@ def get_ready_condition(service):
 
 def get_latest_running_revision(service_name: str, namespace: str = "default"):
     service = get_knative_service(service_name, namespace)
+    if service is None:
+        return None
     latest_revision_name = service['status'].get('latestReadyRevisionName', None)
     return latest_revision_name
 
@@ -285,6 +359,8 @@ def get_pods_for_service(service_name: str, namespace: str = "default"):
     k8s_api_client = config.new_client_from_config(context=context_name)
     api = client.CoreV1Api(k8s_api_client)
     latest_revision = get_latest_running_revision(service_name, namespace)
+    if latest_revision is None:
+        return None
     pods = api.list_namespaced_pod(namespace=namespace,
                                    label_selector=f"serving.knative.dev/service={service_name},serving.knative.dev/revision={latest_revision}")
     return pods
